package game;

import game.exceptions.*;
import game.objects.*;
import game.rmi.RmiPlayerInterface;
import game.rmi.RmiServerInterface;

import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Hashtable;
import java.util.Vector;

public class JspPlayer extends UnicastRemoteObject implements RmiPlayerInterface, AbstractPlayer
{
	private String
		server,
		name,
		password;

	private RmiServerInterface rmiServerInterface;
	private Hashtable<String, AdversaryInfo> adversariesInfo;
	private Vector<Message> mainChat;
	private Hashtable<String, Rank> ranking;
	private boolean loggedIn;

	static
	{
		System.setProperty("java.security.policy", "policy.all");
		System.setSecurityManager(new RMISecurityManager());
	}

	public String toString()
	{
		return name + ": " + super.toString();
	}

	public JspPlayer() throws RemoteException
	{
		super();
		ranking = new Hashtable<String, Rank>();
		mainChat = new Vector<Message>();
		adversariesInfo = new Hashtable<String, AdversaryInfo>();
	}

	public void setServer(String server)
	{
		this.server = server;
	}

	public void setName(String name)
	{
		this.name = name;
	}

	public void setPassword(String password)
	{
		this.password = password;
	}

	public void login() throws NotBoundException, LoginException, RemoteException
	{
		getRmiServerInterface().login(this, name, password, "jsp");
		loggedIn = true;
	}

	public void register() throws NotBoundException, RegisterException, RemoteException
	{
		getRmiServerInterface().register(this, name, password, "jsp");
		loggedIn = true;
	}

	public String getName()
	{
		return name;
	}

	public void logout() throws RemoteException
	{
		getRmiServerInterface().logout(name);
		loggedIn = false;
	}

	public Vector<Message> getGameChat(String adversaryName) throws AdversaryLoggedOffException
	{
		return adversariesInfo.get(adversaryName).getGameChat();
	}

	public void gameChat(Message message) throws RemoteException, AdversaryLoggedOffException
	{
		getRmiServerInterface().gameChat(message);
		adversariesInfo.get(message.getReceiver()).getGameChat().add(message);
	}

	public void beGameChatted(Message message) throws RemoteException
	{
		adversariesInfo.get(message.getSender()).getGameChat().add(message);
	}


	public void beUpdated(LoggedPlayer[] loggedPlayers, Hashtable<String, Rank> ranking)
	{
		String adversaryName;
		for(LoggedPlayer loggedPlayer : loggedPlayers)
			if (!adversariesInfo.containsKey(adversaryName = loggedPlayer.getUserName()))
				adversariesInfo.put(adversaryName, new AdversaryInfo(loggedPlayer));

		adversariesInfo.remove(name);

		this.ranking = ranking;
	}

	public void beMainChatted(Message message) throws RemoteException
	{
		mainChat.add(message);
	}

	public void beInvited(String inviter) throws RemoteException
	{
		adversariesInfo.get(inviter).status = AdversaryStatus.INVITING_ME;
	}

	public void seeInvitationAnswered(String invited, boolean accept) throws RemoteException
	{
		adversariesInfo.get(invited).status = accept ? AdversaryStatus.ACCEPTED_ME : AdversaryStatus.REFUSED_ME;
	}

	public void adversaryDrop(String adversaryName, int columnIndex) throws RemoteException
	{
		try
		{
			adversariesInfo.get(adversaryName).drop(false, columnIndex);
		}
		catch (Victory victory)
		{
			// todo victory: celebrate
		}
		catch (GameResult gameResult)
		{
			rmiServerInterface.endGame(name, gameResult);
		}
	}

	public void startGame(String adversaryName, boolean myTurn) throws RemoteException
	{
		adversariesInfo.get(adversaryName).startGame(myTurn);
	}

	public void endGame(GameResult gameResult) throws RemoteException
	{
		adversariesInfo.get(gameResult.getAdversaryName()).setGameResult(gameResult);
	}

	public Hashtable<String, Rank> getRanking()
	{
		return ranking;
	}

	public Vector<Message> getMainChat()
	{
		return mainChat;
	}

	public void mainChat(Message message) throws RemoteException
	{
		getRmiServerInterface().mainChat(message);
	}

	public RmiServerInterface getRmiServerInterface()
	{
		if (rmiServerInterface == null)
			try
			{
				rmiServerInterface = (RmiServerInterface) Naming.lookup("rmi://" + server + "/Servidor4");
			}
			catch (NotBoundException e)
			{
				//nothing: no d, no d
			}
			catch (MalformedURLException e)
			{
				//nothing: no d, no d
			}
			catch (RemoteException e)
			{
				//nothing: no d, no d
			}

		return rmiServerInterface;
	}

	public void invite(String invitee) throws RemoteException, AdversaryLoggedOffException
	{
		getRmiServerInterface().invite(name, invitee);
		adversariesInfo.get(invitee).status = AdversaryStatus.INVITED_BY_ME;
	}

	public void answerInvitation(String inviter, boolean accept) throws RemoteException, AdversaryLoggedOffException
	{
		getRmiServerInterface().answerInvitation(name, inviter, accept);
	}

	public int getPoints()
	{
		return getPoints(name);
	}

	public int getPoints(String playerName)
	{
		for(Rank rank : ranking.values())
			if (rank.getPlayerName().equals(playerName))
				return rank.getPoints();

		return 0;
	}

	public void drop(String adversaryName, int columnIndex) throws RemoteException, AdversaryLoggedOffException
	{
		getRmiServerInterface().drop(name, adversaryName, columnIndex);
		try
		{
			adversariesInfo.get(adversaryName).drop(true, columnIndex);
		}
		catch (GameResult gameResult)
		{
			//todo
		}
	}

	public void setPlayerName(String playerName)
	{
		setName(playerName);
	}

	public boolean isLoggedIn()
	{
		return loggedIn;
	}

	public AdversaryInfo getAdversaryInfo(String adversaryName)
	{
		return adversariesInfo.get(adversaryName);
	}

	public Hashtable<String, AdversaryInfo> getAdversariesInfo()
	{
		return adversariesInfo;
	}
}